@@ -2,98 +2,72 @@ package Alien::Libarchive::Installer;
 
 use strict;
 use warnings;
+use Role::Tiny::With;
+use Alien::Install::Util;
+use Carp qw( carp );
 
 # ABSTRACT: Installer for libarchive
-our $VERSION = '0.07_01'; # VERSION
+our $VERSION = '0.08_01'; # VERSION
 
+if($^O eq 'MSWin32' && do { require Config; $Config::Config{cc} =~ /cl(\.exe)?$/i })
+{
+  with qw(
+    Alien::Install::Role::Installer
+    Alien::Install::Role::HTTP
+    Alien::Install::Role::Tar 
+    Alien::Install::Role::CMake
+    Alien::Install::Role::TestCompileRun
+    Alien::Install::Role::TestFFI
+  );
+}
+else
+{
+  with qw(
+    Alien::Install::Role::Installer
+    Alien::Install::Role::HTTP
+    Alien::Install::Role::Tar 
+    Alien::Install::Role::Autoconf
+    Alien::Install::Role::TestCompileRun
+    Alien::Install::Role::TestFFI
+  );
+  
+  register_build_requires 'PkgConfig'   => '0.07620' if $^O eq 'MSWin32';
+}
+
+
+use constant versions_url => 'http://www.libarchive.org/downloads/';
+
+sub versions_process
+{
+  sub {
+    my($content) = @_;
+    my @versions;
+    push @versions, [$1,$2,$3] while $content =~ /libarchive-([1-9][0-9]*)\.([0-9]+)\.([0-9]+)\.tar.gz/g;
+    @versions;
+  }
+}
+
+sub versions_sort
+{
+  shift; # $class
+  map { join '.', @$_ } sort { $a->[0] <=> $b->[0] || $a->[1] <=> $b->[1] || $a->[2] <=> $b->[2] } @_;
+}
 
 sub versions_available
 {
-  require HTTP::Tiny;
-  my $url = "http://www.libarchive.org/downloads/";
-  my $response = HTTP::Tiny->new->get($url);
-  
-  die sprintf("%s %s %s", $response->{status}, $response->{reason}, $url)
-    unless $response->{success};
-
-  my @versions;
-  push @versions, [$1,$2,$3] while $response->{content} =~ /libarchive-([1-9][0-9]*)\.([0-9]+)\.([0-9]+)\.tar.gz/g;
-  @versions = map { join '.', @$_ } sort { $a->[0] <=> $b->[0] || $a->[1] <=> $b->[1] || $a->[2] <=> $b->[2] } @versions;
+  carp "versions_available is deprecated, please use versions instead";
+  shift->versions(@_);
 }
 
 
-sub fetch
+sub fetch_url
 {
-  my($class, %options) = @_;
-  
-  my $dir = $options{dir} || eval { require File::Temp; File::Temp::tempdir( CLEANUP => 1 ) };
-
-  require HTTP::Tiny;  
-  my $version = $options{version} || do {
-    my @versions = $class->versions_available;
-    die "unable to determine latest version from listing"
-      unless @versions > 0;
-    $versions[-1];
-  };
-
-  if(defined $ENV{ALIEN_LIBARCHIVE_INSTALL_MIRROR})
-  {
-    my $fn = File::Spec->catfile($ENV{ALIEN_LIBARCHIVE_INSTALL_MIRROR}, "libarchive-$version.tar.gz");
-    return wantarray ? ($fn, $version) : $fn;
-  }
-
-  my $url = "http://www.libarchive.org/downloads/libarchive-$version.tar.gz";
-  
-  my $response = HTTP::Tiny->new->get($url);
-  
-  die sprintf("%s %s %s", $response->{status}, $response->{reason}, $url)
-    unless $response->{success};
-  
-  require File::Spec;
-  
-  my $fn = File::Spec->catfile($dir, "libarchive-$version.tar.gz");
-  
-  open my $fh, '>', $fn;
-  binmode $fh;
-  print $fh $response->{content};
-  close $fh;
-  
-  wantarray ? ($fn, $version) : $fn;
-}
-
-
-sub build_requires
-{
-  my %prereqs = (
-    'HTTP::Tiny'   => 0,
-    'Archive::Tar' => 0,
-  );
-  
-  if($^O eq 'MSWin32')
-  {
-    require Config;
-    if($Config::Config{cc} =~ /cl(\.exe)?$/i)
-    {
-      $prereqs{'Alien::CMake'} = '0.05';
-    }
-    else
-    {
-      $prereqs{'Alien::MSYS'} = '0.07';
-      $prereqs{'PkgConfig'}   = '0.07620';
-    }
-  }
-  
-  \%prereqs;
-}
-
-
-sub system_requires
-{
-  my %prereqs = ();
-  \%prereqs;
+  my(undef, $version) = @_;
+  "http://www.libarchive.org/downloads/libarchive-$version.tar.gz";
 }
 
 
+# TODO: move to Alien::Install::*
 sub system_install
 {
   my($class, %options) = @_;
@@ -107,11 +81,10 @@ sub system_install
   {
     my $alien = Alien::Libarchive->new;
     
-    require File::Spec;
     my $dir;
     my(@dlls) = map { 
-      my($v,$d,$f) = File::Spec->splitpath($_); 
-      $dir = [$v,File::Spec->splitdir($d)]; 
+      my($v,$d,$f) = splitpath $_; 
+      $dir = [$v,splitdir $d]; 
       $f;
     } $alien->dlls;
     
@@ -120,7 +93,7 @@ sub system_install
       libs    => [$alien->libs],
       dll_dir => $dir,
       dlls    => \@dlls,
-      prefix  => File::Spec->rootdir,
+      prefix  => rootdir,
     }, $class;
     eval {
       $build->test_compile_run || die $build->error if $options{test} =~ /^(compile|both)$/;
@@ -172,9 +145,8 @@ sub system_install
         {
           next unless $file =~ /^libarchive\.(dylib|so(\.[0-9]+)*)$/;
         }
-        require File::Spec;
-        my($v,$d) = File::Spec->splitpath($dir, 1);
-        $build->{dll_dir} = [File::Spec->splitdir($d)];
+        my($v,$d) = splitpath $dir, 1;
+        $build->{dll_dir} = [splitdir $d];
         $build->{prefix}  = $v;
         $build->{dlls}    = [$file];
         closedir $dh;
@@ -222,193 +194,80 @@ sub _try_pkg_config
   [Text::ParseWords::shellwords($value)];
 }
 
-sub _msys
+sub build_install_cflags
 {
-  my($sub) = @_;
+  my(undef, $prefix) = @_;
+  my $pkg_config_dir = catdir $prefix, 'lib', 'pkgconfig';
+  my @flags = _try_pkg_config($pkg_config_dir, 'cflags', '-I' . catdir($prefix, 'include'), '--static');
+  push @flags, '-DLIBARCHIVE_STATIC' if $^O =~ /^(cygwin|MSWin32)$/;
+  @flags;
+}
+
+sub build_install_libs
+{
+  my(undef, $prefix) = @_;
+  my $pkg_config_dir = catdir $prefix, 'lib', 'pkgconfig';
+  _try_pkg_config($pkg_config_dir, 'libs',   '-L' . catdir($prefix, 'lib'), '--static');
+}
+
+sub build_install_dlls
+{
+  my(undef, $dir) = @_;
+  opendir(my $dh, $dir);
+  [grep { ! -l catfile $dir, $_ } grep { /\.so/ || /\.(dll|dylib)$/ } grep !/^\./, readdir $dh];
+}
+
+register_hook post_install => sub {
+  my($class, $prefix) = @_;
+
+  my $pkg_config_dir = catdir $prefix, 'lib', 'pkgconfig';
+  my $pcfile = catfile $pkg_config_dir, 'libarchive.pc';
+    
+  my @content;
   require Config;
-  if($^O eq 'MSWin32')
+  if($Config::Config{cc} !~ /cl(\.exe)?$/i)
   {
-    if($Config::Config{cc} !~ /cl(\.exe)?$/i)
-    {
-      require Alien::MSYS;
-      return Alien::MSYS::msys(sub{ $sub->('make') });
-    }
+    open my $fh, '<', $pcfile;
+    @content = map { s{$prefix}{'${pcfiledir}/../..'}eg; $_ } do { <$fh> };
+    close $fh;
+  }
+  else
+  {
+    # TODO: later when we know the version with more
+    # certainty, we can update this file with the
+    # Version
+    @content = join "\n", "prefix=\${pcfiledir}/../..",
+                          "exec_prefix=\${prefix}",
+                          "libdir=\${exec_prefix}/lib",
+                          "includedir=\${prefix}/include",
+                          "Name: libarchive",
+                          "Description: library that can create and read several streaming archive formats",
+                          "Cflags: -I\${includedir}",
+                          "Libs: advapi32.lib \${libdir}/archive_static.lib",
+                          "Libs.private: ",
+                          "";
+    require File::Path;
+    File::Path::mkpath($pkg_config_dir, 0, 0755);
   }
-  $sub->($Config::Config{make});
-}
-
-sub build_install
-{
-  my($class, $prefix, %options) = @_;
-  
-  $options{test} ||= 'compile';
-  die "test must be one of compile, ffi or both"
-    unless $options{test} =~ /^(compile|ffi|both)$/;
-  die "need an install prefix" unless $prefix;
-  
-  $prefix =~ s{\\}{/}g;
-  
-  my $dir = $options{dir} || do { require File::Temp; File::Temp::tempdir( CLEANUP => 1 ) };
-  
-  require Archive::Tar;
-  my $tar = Archive::Tar->new;
-  $tar->read($options{tar} || $class->fetch);
-  
-  require Cwd;
-  my $save = Cwd::getcwd();
-  
-  chdir $dir;  
-  my $build = eval {
-  
-    $tar->extract;
-
-    chdir do {
-      opendir my $dh, '.';
-      my(@list) = grep !/^\./,readdir $dh;
-      close $dh;
-      die "unable to find source in build root" if @list == 0;
-      die "confused by multiple entries in the build root" if @list > 1;
-      $list[0];
-    };
-  
-    _msys(sub {
-      my($make) = @_;
-      require Config;
-      if($Config::Config{cc} !~ /cl(\.exe)?$/i)
-      {
-        system 'sh', 'configure', "--prefix=$prefix", '--with-pic';
-        die "configure failed" if $?;
-      }
-      else
-      {
-        require Alien::CMake;
-        my $cmake = Alien::CMake->config('prefix') . '/bin/cmake.exe';
-        my $system = $make =~ /nmake(\.exe)?$/ ? 'NMake Makefiles' : 'MinGW Makefiles';
-        system $cmake,
-          -G => $system,
-          "-DCMAKE_MAKE_PROGRAM:PATH=$make",
-          "-DCMAKE_INSTALL_PREFIX:PATH=$prefix",
-          "-DENABLE_TEST=OFF",
-          ".";
-        die "cmake failed" if $?;
-      }
-      system $make, 'all';
-      die "make all failed" if $?;
-      system $make, 'install';
-      die "make install failed" if $?;
-    });
-
-    require File::Spec;
-
-    foreach my $name ($^O =~ /^(MSWin32|cygwin)$/ ? ('bin','lib') : ('lib'))
-    {
-      do {
-        my $static_dir = File::Spec->catdir($prefix, $name);
-        my $dll_dir    = File::Spec->catdir($prefix, 'dll');
-        require File::Path;
-        File::Path::mkpath($dll_dir, 0, 0755);
-        my $dh;
-        opendir $dh, $static_dir;
-        my @list = readdir $dh;
-        @list = grep { /\.so/ || /\.(dylib|la|dll|dll\.a)$/ } grep !/^\./, @list;
-        closedir $dh;
-        foreach my $basename (@list)
-        {
-          my $from = File::Spec->catfile($static_dir, $basename);
-          my $to   = File::Spec->catfile($dll_dir,    $basename);
-          if(-l $from)
-          {
-            symlink(readlink $from, $to);
-            unlink($from);
-          }
-          else
-          {
-            require File::Copy;
-            File::Copy::move($from, $to);
-          }
-        }
-      };
-    }
-
-    my $pkg_config_dir = File::Spec->catdir($prefix, 'lib', 'pkgconfig');
-    
-    my $pcfile = File::Spec->catfile($pkg_config_dir, 'libarchive.pc');
-    
-    do {
-      my @content;
-      if($Config::Config{cc} !~ /cl(\.exe)?$/i)
-      {
-        open my $fh, '<', $pcfile;
-        @content = map { s{$prefix}{'${pcfiledir}/../..'}eg; $_ } do { <$fh> };
-        close $fh;
-      }
-      else
-      {
-        # TODO: later when we know the version with more
-        # certainty, we can update this file with the
-        # Version
-        @content = join "\n", "prefix=\${pcfiledir}/../..",
-                              "exec_prefix=\${prefix}",
-                              "libdir=\${exec_prefix}/lib",
-                              "includedir=\${prefix}/include",
-                              "Name: libarchive",
-                              "Description: library that can create and read several streaming archive formats",
-                              "Cflags: -I\${includedir}",
-                              "Libs: advapi32.lib \${libdir}/archive_static.lib",
-                              "Libs.private: ",
-                              "";
-        require File::Path;
-        File::Path::mkpath($pkg_config_dir, 0, 0755);
-      }
       
-      my($version) = map { /^Version:\s*(.*)$/; $1 } grep /^Version: /, @content;
-      # older versions apparently didn't include the necessary -I and -L flags
-      if(defined $version && $version =~ /^[12]\./)
-      {
-        for(@content)
-        {
-          s/^Libs: /Libs: -L\${libdir} /;
-        }
-        push @content, "Cflags: -I\${includedir}\n";
-      }
-      
-      open my $fh, '>', $pcfile;
-      print $fh @content;
-      close $fh;
-    };
-    
-    my $build = bless {
-      cflags  => _try_pkg_config($pkg_config_dir, 'cflags', '-I' . File::Spec->catdir($prefix, 'include'), '--static'),
-      libs    => _try_pkg_config($pkg_config_dir, 'libs',   '-L' . File::Spec->catdir($prefix, 'lib'),     '--static'),
-      prefix  => $prefix,
-      dll_dir => [ 'dll' ],
-      dlls    => do {
-        opendir(my $dh, File::Spec->catdir($prefix, 'dll'));
-        [grep { ! -l File::Spec->catfile($prefix, 'dll', $_) } grep { /\.so/ || /\.(dll|dylib)$/ } grep !/^\./, readdir $dh];
-      },
-    }, $class;
-    
-    if($^O eq 'cygwin' || $^O eq 'MSWin32')
+  my($version) = map { /^Version:\s*(.*)$/; $1 } grep /^Version: /, @content;
+  # older versions apparently didn't include the necessary -I and -L flags
+  if(defined $version && $version =~ /^[12]\./)
+  {
+    for(@content)
     {
-      # TODO: should this go in the munged pc file?
-      unshift @{ $build->{cflags} }, '-DLIBARCHIVE_STATIC';
+      s/^Libs: /Libs: -L\${libdir} /;
     }
-
-    $build->test_compile_run || die $build->error if $options{test} =~ /^(compile|both)$/;
-    $build->test_ffi         || die $build->error if $options{test} =~ /^(ffi|both)$/;
-    $build;
-  };
-  
-  my $error = $@;
-  chdir $save;
-  die $error if $error;
-  $build;
-}
+    push @content, "Cflags: -I\${includedir}\n";
+  }
+      
+  open my $fh, '>', $pcfile;
+  print $fh @content;
+  close $fh;
+};
 
 
-sub cflags  { shift->{cflags}  }
-sub libs    { shift->{libs}    }
-sub version { shift->{version} }
+# TODO: move to Alien::Install::*
 
 sub dlls
 {
@@ -436,8 +295,7 @@ sub dlls
       $self->{libs} = [ $self->{libs} ] unless ref $self->{libs};
       my $path = DynaLoader::dl_findfile(grep /^-l/, @{ $self->libs });
       die "unable to find dynamic library" unless defined $path;
-      require File::Spec;
-      my($vol, $dirs, $file) = File::Spec->splitpath($path);
+      my($vol, $dirs, $file) = splitpath $path;
       if($^O eq 'openbsd')
       {
         # on openbsd we get the .a file back, so have to scan
@@ -451,133 +309,45 @@ sub dlls
         $self->{dlls} = [ $file ];
       }
       $self->{dll_dir} = [];
-      $self->{prefix} = $prefix = File::Spec->catpath($vol, $dirs);
+      $self->{prefix} = $prefix = catpath($vol, $dirs);
     }
   }
   
-  require File::Spec;
-  map { File::Spec->catfile($prefix, @{ $self->{dll_dir} }, $_ ) } @{ $self->{dlls} };
+  map { catfile $prefix, @{ $self->{dll_dir} }, $_  } @{ $self->{dlls} };
 }
 
 
-sub test_compile_run
-{
-  my($self, %opt) = @_;
-  delete $self->{error};
-  $self->{quiet} = 1 unless defined $self->{quiet};
-  my $cbuilder = $opt{cbuilder} || do { require ExtUtils::CBuilder; ExtUtils::CBuilder->new(quiet => $self->{quiet}) };
-  
-  unless($cbuilder->have_compiler)
-  {
-    $self->{error} = 'no compiler';
-    return;
-  }
-  
-  require File::Spec;
-  my $dir = $opt{dir} || do { require File::Temp; File::Temp::tempdir( CLEANUP => 1 ) };
-  my $fn = File::Spec->catfile($dir, 'test.c');
-  do {
-    open my $fh, '>', $fn;
-    print $fh "#include <archive.h>\n",
-              "#include <archive_entry.h>\n",
-              "#include <stdio.h>\n",
-              "int\n",
-              "main(int argc, char *argv[])\n",
-              "{\n",
-              "  printf(\"version = '%d'\\n\", archive_version_number());\n",
-              "  return 0;\n",
-              "}\n";
-    close $fh;
-  };
-  
-  my $test_object = eval {
-    $cbuilder->compile(
-      source               => $fn,
-      extra_compiler_flags => $self->{cflags} || [],
-    );
-  };
-  
-  if(my $error = $@)
-  {
-    $self->{error} = $error;
-    return;
-  }
-  
-  my $test_exe = eval {
-    $cbuilder->link_executable(
-      objects            => $test_object,
-      extra_linker_flags => $self->{libs} || [],
-    );
-  };
-  
-  if(my $error = $@)
-  {
-    $self->{error} = $error;
-    return;
-  }
-  
-  if($test_exe =~ /\s/)
-  {
-    $test_exe = Win32::GetShortPathName($test_exe) if $^O eq 'MSWin32';
-    $test_exe = Cygwin::win_to_posix_path(Win32::GetShortPathName(Cygwin::posix_to_win_path($test_exe))) if $^O eq 'cygwin';
-  }
-  
-  my $output = `$test_exe`;
-  
-  if($? == -1)
-  {
-    $self->{error} = "failed to execute $!";
-    return;
-  }
-  elsif($? & 127)
-  {
-    $self->{error} = "child died with siganl " . ($? & 127);
-    return;
-  }
-  elsif($?)
-  {
-    $self->{error} = "child exited with value " . ($? >> 8);
-    return;
-  }
-  
-  if($output =~ /version = '([0-9]+)([0-9]{3})([0-9]{3})'/)
-  {
-    return $self->{version} = join '.', map { int } $1, $2, $3;
-  }
-  else
-  {
-    $self->{error} = "unable to retrieve version from output";
-    return;
-  }
-}
+use constant test_compile_run_program =>
+  join ("\n",
+    "#include <archive.h>",
+    "#include <archive_entry.h>",
+    "#include <stdio.h>",
+    "int",
+    "main(int argc, char *argv[])",
+    "{",
+    "  printf(\"version = '%s'\\n\", archive_version_string());",
+    "  return 0;",
+    "}",
+    "",
+  );
+;
+
+use constant test_compile_run_match => qr{version = 'libarchive (.*?)'};
 
 
-sub test_ffi
+sub test_ffi_signature
 {
-  my($self) = @_;
   require FFI::Raw;
-  delete $self->{error};
-
-  foreach my $dll ($self->dlls)
-  {
-    my $archive_version_number = eval {
-      FFI::Raw->new(
-        $dll, 'archive_version_number',
-        FFI::Raw::int(),
-      );
-    };
-    next if $@;
-    if($archive_version_number->() =~ /^([0-9]+)([0-9]{3})([0-9]{3})/)
-    {
-      return $self->{version} = join '.', map { int } $1, $2, $3;
-    }
-  }
-  $self->{error} = 'could not find archive_version_number';
-  return; 
+  ('archive_version_number', FFI::Raw::int());
 }
 
+sub test_ffi_version
+{
+  my(undef, $function) = @_;
+  return join '.', map { int } $1, $2, $3 if $function->() =~ /^([0-9]+)([0-9]{3})([0-9]{3})/;
+  return;
+}
 
-sub error { shift->{error} }
 
 1;
 
@@ -593,7 +363,7 @@ Alien::Libarchive::Installer - Installer for libarchive
 
 =head1 VERSION
 
-version 0.07_01
+version 0.08_01
 
 =head1 SYNOPSIS
 
@@ -675,9 +445,9 @@ an instance of L<Alien::Libarchive::Installer> which can be
 queried to retrieve the settings needed to interact with 
 libarchive via XS or L<FFI::Raw>.
 
-=head2 versions_available
+=head2 versions
 
- my @versions = Alien::Libarchive::Installer->versions_available;
+ my @versions = Alien::Libarchive::Installer->versions;
  my $latest_version = $versions[-1];
 
 Return the list of versions of libarchive available on the Internet.
@@ -796,9 +566,9 @@ These options may be passed into build_install:
 
 =over 4
 
-=item tar
+=item archive
 
-Filename where the libarchive source tar is located.
+Filename where the libarchive source tarball is located.
 If not specified the latest version will be downloaded
 from the Internet.
 
